function [ta,xa] = eulersol(func,tinp,x0)
% Lsung einer Differentialgleichung mit der Eulermethode.
% Berechnung und Ausgabe nur an den Werten von tinp.
% ACHTUNG: Keine Genauigkeitsberprfung.
% Input:  func  String mit Funktionsnamen fr Funktion zur Berechnung von dx/dt,
%                xdot = feval(func,t,x) bedeutet z.B xdot = pendeleq(t,x)
%                wenn func den Wert 'pendeleq' hat.
%         tinp  Zeitwerte
%         x0    Startwerte
% Output: ta    Zeitvektor (Spaltenvektor)
%         xa    Array mit gleich vielen Zeilen wie Zeitvektor ta
%                Zeilen enthalten x(1), x(2), ... x(n) 
%                n - Lnge von x0
%
% Autor: Winfried Kernbichler     23.01.2000

lt = length(tinp); % Lnge des Zeitvektors
lx = length(x0);   % Lnge des Startvektors
ta = zeros(lt,1);  % Richtige Gre und Ausrichtung des Zeitvektors 
xa = zeros(lt,lx); % und des xa-Arrays

% Stellt sicher das Startwert ein Zeilenvektor ist
[nx,mx] = size(x0); if nx>mx, x0 = x0'; end; 

x = x0; xa(1,:) = x0;           % Anfangswert
for it = 2:lt                   % Abarbeiten aller Zeitschritte
   t  = tinp(it);               % Zeit
   dt = tinp(it) - tinp(it-1);  % Zeitdifferenz zum vorigen Schritt
   xdot = feval(func,t,x);      % Auswertung der Funktion func
   x = x + xdot*dt;             % Eulerschritt x(t+dt) = x(t) + (dx/dt) * dt
   ta(it)   = t;                % Zuweisen der Outputgren fr ta
   xa(it,:) = x;                %  und xa
end   