% Script htlinearfit.m
% Zeigt einige lineare Fits.
%
% Autor: Winfried Kernbichler 12.12.1999

echo off; clc;
% Definitionen: Frequenz und Gre der Strung
nu = 1;
om = 2*pi * nu;
errlev = 0.3;
% Zeitvektor
ts = 1; te = 10; tanz = 100;
t = linspace(ts,te,tanz);

% Darzustellende Funktion
% f = a1 + a2*sin(om*t) + a3*cos(om*t)/t

% Summanden der Funktion ohne Konstanten
f1 = ones(size(t));
f2 = sin(om*t);
f3 = cos(om*t) ./ t;

M = [f1.', f2.', f3.']; % Matrix mit drei Spalten

% Konstanten a1, a2 und a3 in der Variablen a
a = [1.0 0.5 5.0];

% Fr Matrixmultiplikation braucht man einen Spaltenvektor
A = a.';
T = t.';

% Matrixmultiplikation
% In jeder Zeile von F steht ein Funktionswert
F = M*A;

% Verrauschen der Daten
FR = F + (rand(size(F))-0.5)*errlev; 

% Plot
fig1 = figure(1); clf;
plot(T,F,'b',T,FR,'bo'); 
title('f = a_1 + a_2 sin({\omega}t) + a_3 cos({\omega}t)/t');
xlabel('Time'); ylabel('F');

% Least Squares Fit 
% berbestimmtes Gleichungssystem
AR = M \ FR; % Neue Koeffizienten 
FF = M * AR; % Funktionswerte mit neuen Koeffizienten

hold on;
plot(T,FF,'r'); legend('F','F random','F fit');
hold off;

disp(' '); disp('Linearer Fit mit Winkelfunktionen'); disp(' ');
disp('Original Koeffizienten');
disp(A');
disp('Fit Koeffizienten');
disp(AR');
disp('PAUSE MODE'); pause;

% Fit mit Exponentialfunktion
%  Ursprngliche Funktion
%  g = b1*exp(-b2*t))*t^(b3)
%
%  Durch Logarithmieren erhlt man:
%  ln(g) = ln(b1) - b2*t + b3*ln(t)
%  g = exp(ln(g))

% Erstellen der Matrix
g1 = ones(size(t));
g2 = - t;
g3 = log(t);
MAT = [g1.', g2.', g3.'];

% Paramterwerte b
b = [log(4) 1 0.5];
B = b.';

% Berechnen der Funktionswerte
LOGG = MAT * B;
G = exp(LOGG);

% Verrauschen der Logarithmischen Daten
LOGGR = LOGG + (rand(size(LOGG))-0.5)*errlev; 
GR = exp(LOGGR);
% Neue Koeffizienten (Fit)
BR = MAT \ LOGGR;

% Neue Funktionswerte
LOGGF = MAT * BR; 
GF = exp(LOGGF);

% Darstellung Logarithmisch
fig2 = figure(2); clf;
subplot(1,2,1);
loglog(T,G,'b',T,GR,'bo',T,GF,'r'); title('Data');
xlabel('Time'); ylabel('G');
grid on;

% und linear
subplot(1,2,2);
plot(T,G,'b',T,GR,'bo',T,GF,'r'); title('Data');
xlabel('Time'); ylabel('G');
grid on;

disp(' '); disp('Exponentialfit'); disp(' ');
disp('Original Koeffizienten');
disp(B');
disp('Fit Koeffizienten');
disp(BR');
disp('PAUSE MODE'); pause;


% Polynomfit von Sinus zwischen 0 und pi

% Daten (etwas ber den Bereich hinaus)
xextra = pi/20;
xx = linspace(0-xextra,pi+xextra,100);
yy = sin(xx);

% Polynomfit dritten Grades
pol1 = polyfit(xx,yy,3);

% Fit mit gleich vielen Koeffizienten aber nur ungerade Funktionen
SM = [(xx.^7).' , (xx.^5).' , (xx.^3).' , xx.'];
YY = yy.';
SA = SM \ YY;

% Rckfhrung in Polynomschreibweise
pol2 = zeros(1,8);
pol2(1) = SA(1);
pol2(3) = SA(2);
pol2(5) = SA(3);
pol2(7) = SA(4);

% Darstellung
fig3 = figure(3);
subplot(2,1,1);
plot(xx,yy,'ko',xx,polyval(pol1,xx),'b',xx,polyval(pol2,xx),'r');
legend('Data','degree 3','odd 7');
title('Fit for sin(t)');

% Darstellung der Residuals
subplot(2,1,2)
plot(xx,yy-polyval(pol1,xx),'b',xx,yy-polyval(pol2,xx),'r');
legend('degree 3','odd 7');
title('Residuals');

disp(' '); disp('PAUSE MODE - Zum Beenden Taste drcken'); pause;
close(fig1);
close(fig2);
close(fig3);


