function xdot = pendeleq(t,x)
% Unterprogramm zur Berechnung der exakten und der lineare Pendeldifferentialgleichungen
%
% Input Parameter:  t       Zeit
%                   x       Vektor mit [phi omega]
% Input Global:     OM2     Quadrat der Grundfrequenz
%                   KAPPA   Dmpfungsfaktor
%                   METHOD  Methode (exact oder linear)
%
% Output:           xdot    Vektor mit [dphi/dt domega/dt]; selbe Groesse wie x
%
% Autor: Winfried Kernbichler    23.01.2000
global OM2 KAPPA METHOD F0 FOMEGA FPHASE

xdot = zeros(size(x)); % Gleiche Gre und Ausrichtung wie x
switch METHOD
case 'e'  % Exakte Differentialgleichung
   xdot(:) = [ x(2) , -KAPPA*x(2)-OM2*sin(x(1)) + F0*sin(FOMEGA*t+FPHASE) ];
case 'l'  % Lineare Differentialgleichung
   xdot(:) = [ x(2) , -KAPPA*x(2)-OM2*x(1) + F0*sin(FOMEGA*t+FPHASE) ];
end
